@@ -36,7 +36,6 @@
 #include "armour.h"
 #include "cryptmount.h"
 #include "dmutils.h"
-#include "looputils.h"
 #include "utils.h"
 #ifdef TESTING
 #  include "cmtesting.h"
@@ -46,6 +45,10 @@
  *  @{ */
 
 
+#ifndef GCRYPT_REQ_VERSION
+#  define GCRYPT_REQ_VERSION "1.1.42"
+#endif
+
 typedef struct {
     unsigned keyslot;
 } luks_overrides_t;
@@ -58,14 +61,7 @@ typedef struct {
 #if USE_LUKSCOMPAT
 #  if !USE_MODULES || defined(AS_MODULE)
 #    include <gcrypt.h>
-#    include "luks/crypto_backend.h"
-#    include "luks/internal.h"
-#    include "luks/luks.h"
-#    if HAVE_UUID
-#      include <uuid.h>
-#    endif
-
-extern struct setup_backend setup_libdevmapper_backend;
+#    include <libcryptsetup.h>
 
 #if defined(TESTING) && defined(AS_MODULE)
 cm_testinfo_t *test_ctxtptr;
@@ -74,16 +70,17 @@ cm_testinfo_t *test_ctxtptr;
 
 static int kmluks_hdrvalid(FILE *fp_key)
     /* Check whether a valid LUKS header is present */
-{   const unsigned char luks_magic[] = LUKS_MAGIC;
+{   const unsigned char luks_magic[] = { 'L','U','K','S', 0xba, 0xbe };
+    const size_t magic_len = sizeof(luks_magic);
     char buff[32];
     int flg = 0;
 
     if (fp_key == NULL) return 0;
 
-    if (cm_fread((void*)buff, (size_t)LUKS_MAGIC_L, fp_key) == 0) {
-        fseek(fp_key, -((long)LUKS_MAGIC_L), SEEK_CUR);
+    if (cm_fread((void*)buff, magic_len, fp_key) == 0) {
+        fseek(fp_key, -((long)magic_len), SEEK_CUR);
         flg = (strncmp(buff, (const char*)luks_magic,
-                (size_t)LUKS_MAGIC_L) == 0);
+                (size_t)magic_len) == 0);
     }
 
     return flg;
@@ -127,9 +124,6 @@ static void kmluks_splitmode(const char *fullname, char **cipher, char **mode)
 static int kmluks_init_algs()
 {   static int done_secmem = 0;
     int flg = 0;
-    const char *GCRYPT_REQ_VERSION="1.1.42";
-
-    flg = crypt_backend_init(NULL) | crypt_random_init(NULL);
 
     if (!done_secmem || !gcry_control(GCRYCTL_INITIALIZATION_FINISHED_P)) {
         if (!gcry_check_version(GCRYPT_REQ_VERSION)) return -1;
@@ -205,35 +199,99 @@ static unsigned kmluks_get_properties(const bound_tgtdefn_t *boundtgt)
 }
 
 
+/*!
+ *  Change the password associated with a given LUKS key-slot.
+ *
+ *  This will either create an entirely new keyslot with the given password,
+ *  or attempt to change the password associated with a particular keyslot
+ *  while taking a temporary backup of the key in that slot. This requires
+ *  that there is at least one spare keyslot available to take that backup.
+ *
+ *  @param key      The volume key for the LUKS device
+ *  @param keyslot  Either CRYPT_ANY_SLOT or a nominated slot.
+ *
+ *  @return The slot associated with the new password.
+ */
+int kmluks_change_slot_passwd(struct crypt_device *cd, int keyslot,
+                              const unsigned char *key, const int keylen,
+                              const char *passwd)
+{   const size_t passwdlen = strlen(passwd);
+    int new_slot = -1, bckp_slot = -1, r;
+    char logmsg[256];
+
+    if (keyslot != CRYPT_ANY_SLOT) {
+        bckp_slot = crypt_keyslot_add_by_volume_key(cd, CRYPT_ANY_SLOT,
+                                                    (const char*)key, keylen,
+                                                    passwd, passwdlen);
+        if (bckp_slot < 0) return bckp_slot;
+        r = crypt_keyslot_destroy(cd, keyslot);
+        if (r < 0) return r;
+
+        sprintf(logmsg, "kmluks created keyslot backup %d -> %d",
+                        keyslot, bckp_slot);
+        crypt_log(cd, CRYPT_LOG_NORMAL, logmsg);
+    }
+
+    new_slot = crypt_keyslot_add_by_volume_key(cd, keyslot,
+                                               (const char*)key, keylen,
+                                               passwd, passwdlen);
+    if (new_slot < 0) return new_slot;
+    sprintf(logmsg, "kmluks added keyslot %d", new_slot);
+    crypt_log(cd, CRYPT_LOG_NORMAL, logmsg);
+
+    if (keyslot != CRYPT_ANY_SLOT
+          && bckp_slot >= 0 && bckp_slot != new_slot) {
+        crypt_keyslot_destroy(cd, bckp_slot);
+
+        sprintf(logmsg, "kmluks removed keyslot backup %d", bckp_slot);
+        crypt_log(cd, CRYPT_LOG_NORMAL, logmsg);
+    }
+
+    return new_slot;
+}
+
+
+void kmluks_log(int level, const char *msg, void *data)
+  /*! stderr-based logging function for libcryptsetup */
+{
+  fprintf(stderr, "LUKS[%d] - %s\n", level, msg);
+}
+
+
 static int kmluks_get_key(bound_tgtdefn_t *boundtgt,
             const km_pw_context_t *pw_ctxt,
             unsigned char **key, int *keylen, FILE *fp_key)
-    /* Extract key from LUKS header file */
+    /*! Extract key from LUKS header file */
 {   tgtdefn_t *tgt = boundtgt->tgt;
-    char *passwd = NULL;
-    const char *tgtdev = NULL;
+    char *passwd = NULL, label[256];
     struct crypt_device *luks_ctxt = NULL;
-    struct luks_phdr *lukshdr;
-    struct volume_key *lukskey = NULL;
-    int killloop = 0, slot, eflag = ERR_NOERROR;
+    int slot = -1, eflag = ERR_NOERROR;
     luks_overrides_t *luksor;
     int64_t delta;
-    const size_t namesz = LUKS_CIPHERNAME_L + LUKS_CIPHERMODE_L + 8;
+    size_t lcs_keylen = 256;
+    const size_t namesz = 128;
+
+    /* This is vulnerable to permission-issues created by libgcrypt
+     *  -- see http://code.google.com/p/cryptsetup/issues/detail?id=47,
+     * which are mitigated by kmluks_init_algs(). */
+
+    snprintf(label, sizeof(label), "cm-luks-tmp-%d-%x",
+             getpid(), (unsigned)(size_t)tgt);
 
     eflag = km_get_passwd(tgt->ident, pw_ctxt, &passwd, 0, 0);
     if (eflag != ERR_NOERROR) goto bail_out;
 
-    if (blockify_file(tgt->key.filename, O_RDONLY, NULL, &tgtdev, &killloop) != ERR_NOERROR) {
-        fprintf(stderr, _("Failed to create loop device for LUKS keyfile\n"));
-        eflag = ERR_BADDEVICE;
+    if (crypt_init(&luks_ctxt, tgt->key.filename) < 0
+        || crypt_load(luks_ctxt, NULL, NULL) < 0) {
+        fprintf(stderr, _("Failed to initialize device for LUKS keyfile\n"));
+        eflag = ERR_BADDECRYPT;
         goto bail_out;
     }
+    //crypt_set_log_callback(luks_ctxt, kmluks_log, NULL);  // FIXME - remove soon
 
-    crypt_init(&luks_ctxt, tgtdev);
-    lukshdr = crypt_get_lukshdr(luks_ctxt);
-    LUKS_read_phdr(lukshdr, 1, 0, luks_ctxt);
-    slot = LUKS_open_key_with_hdr(-1, passwd, strlen(passwd),
-                                lukshdr, &lukskey, luks_ctxt);
+    slot = crypt_activate_by_passphrase(luks_ctxt, label,
+                                        CRYPT_ANY_SLOT, passwd, strlen(passwd),
+                                        CRYPT_ACTIVATE_READONLY);
     if (slot < 0) {
         fprintf(stderr, _("Failed to extract LUKS key for \"%s\" (errno=%d)\n"),
                 tgt->ident, -slot);
@@ -241,8 +299,8 @@ static int kmluks_get_key(bound_tgtdefn_t *boundtgt,
         goto bail_out;
     }
 
-    /* Extract cipher-algorithm parameter from LUKS header: */
-    delta = (lukshdr->payloadOffset - tgt->start);
+    /* Extract cipher-algorithm parameters from LUKS header: */
+    delta = (crypt_get_data_offset(luks_ctxt) - tgt->start);
     if (delta >= 0) {
         tgt->start += delta;
         if (tgt->length >= 0) tgt->length -= delta;
@@ -250,25 +308,25 @@ static int kmluks_get_key(bound_tgtdefn_t *boundtgt,
     if (tgt->cipher != NULL) free((void*)tgt->cipher);
     tgt->cipher = (char*)malloc(namesz);
     snprintf(tgt->cipher, namesz, "%s-%s",
-            lukshdr->cipherName, lukshdr->cipherMode);
-    tgt->ivoffset = 0;
+             crypt_get_cipher(luks_ctxt), crypt_get_cipher_mode(luks_ctxt));
+    tgt->ivoffset = crypt_get_iv_offset(luks_ctxt);
     if (boundtgt->km_data != NULL) free((void*)boundtgt->km_data);
     luksor = (luks_overrides_t*)malloc(sizeof(luks_overrides_t));
     luksor->keyslot = slot;
     boundtgt->km_data = (void*)luksor;
 
     /* Take copy of LUKS master-key: */
-    *keylen = lukskey->keylength;
-    *key = (unsigned char*)sec_realloc((void*)*key, (size_t)lukskey->keylength);
-    memcpy((void*)*key, (const void*)lukskey->key, (size_t)*keylen);
+    *key = (unsigned char*)sec_realloc((void*)*key, lcs_keylen);
+    crypt_volume_key_get(luks_ctxt, slot, (char*)*key, &lcs_keylen,
+                         passwd, strlen(passwd));
+    *keylen = lcs_keylen;
 
   bail_out:
 
-    unblockify_file(&tgtdev, killloop);
+    crypt_deactivate(luks_ctxt, label);
+    crypt_free(luks_ctxt);
     udev_settle();
     if (passwd != NULL) sec_free((void*)passwd);
-    if (lukskey != NULL) crypt_free_volume_key(lukskey);
-    if (luks_ctxt != NULL) crypt_free(luks_ctxt);
 
     return eflag;
 }
@@ -279,17 +337,12 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
             const unsigned char *key, const int keylen, FILE *fp_key)
     /* Store or create key in LUKS header */
 {
-#if HAVE_UUID
     const keyinfo_t *keyinfo = &boundtgt->tgt->key;
     char *passwd = NULL, *ciphername = NULL, *ciphermode = NULL;
-    const char *tgtdev = NULL;
     struct crypt_device *luks_ctxt = NULL;
-    struct luks_phdr *lukshdr;
-    struct volume_key *lukskey = NULL;
     unsigned keyslot = 0;
-    uint64_t *iteration_time = NULL, *PBKDF2_per_sec = NULL;
     luks_overrides_t *luksor = NULL;
-    int formatting = 0, killloop = 0, r, eflag = ERR_NOERROR;
+    int formatting = 0, r, eflag = ERR_NOERROR;
 
     formatting = (boundtgt->km_data == NULL) && !kmluks_hdrvalid(fp_key);
 
@@ -300,7 +353,9 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
     if (formatting) {
 #ifndef TESTING
         char msgbuff[1024];
-        snprintf(msgbuff, sizeof(msgbuff), _("Formatting \"%s\", will probably destroy all existing data"), keyinfo->filename);
+        snprintf(msgbuff, sizeof(msgbuff),
+                _("Formatting \"%s\", will probably destroy all existing data"),
+                keyinfo->filename);
         if (!cm_confirm(msgbuff)) {
             eflag = ERR_ABORT;
             goto bail_out;
@@ -311,80 +366,64 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
     eflag = km_get_passwd(boundtgt->tgt->ident, pw_ctxt, &passwd, 1, 1);
     if (eflag != ERR_NOERROR) goto bail_out;
 
-    eflag = blockify_file(keyinfo->filename, O_RDWR, NULL, &tgtdev, &killloop);
-    if (eflag != ERR_NOERROR) {
-        fprintf(stderr, _("Failed to create loop device for LUKS keyfile\n"));
+    if (crypt_init(&luks_ctxt, keyinfo->filename) < 0) {
+        fprintf(stderr, _("Failed to initialize device for LUKS keyfile\n"));
+        eflag = ERR_BADDECRYPT;
         goto bail_out;
     }
-
-    crypt_init(&luks_ctxt, tgtdev);
-    lukshdr = crypt_get_lukshdr(luks_ctxt);
-    crypt_get_pbkdf2params(luks_ctxt, &iteration_time, &PBKDF2_per_sec);
-    lukskey = crypt_alloc_volume_key(keylen, (const char*)key);
+    //crypt_set_log_callback(luks_ctxt, kmluks_log, NULL);  // FIXME - remove soon
 
     if (formatting) {
-        /* Format new partition */
-        unsigned long reqd_align = DEFAULT_DISK_ALIGNMENT, align_offs = 0;
+        struct crypt_params_luks1 luks_params = { boundtgt->tgt->key.digestalg,
+                                                  0, NULL };
 
-        keyslot = 0;
-        device_topology_alignment(crypt_get_device(luks_ctxt), &reqd_align,
-                                &align_offs, DEFAULT_DISK_ALIGNMENT);
         kmluks_splitmode(boundtgt->tgt->cipher, &ciphername, &ciphermode);
-        r = LUKS_generate_phdr(lukshdr, lukskey, ciphername, ciphermode,
-                            boundtgt->tgt->key.digestalg, NULL, LUKS_STRIPES,
-                            reqd_align / SECTOR_SIZE, align_offs / SECTOR_SIZE,
-                            *iteration_time, PBKDF2_per_sec, 0, luks_ctxt);
+
+        r = crypt_format(luks_ctxt, CRYPT_LUKS1,
+                         ciphername, ciphermode, NULL,
+                         (const char*)key, keylen, &luks_params);
         if (r < 0) {
             fprintf(stderr, _("Failed to create LUKS header for \"%s\"\n"),
                     boundtgt->tgt->ident);
             eflag = ERR_BADDEVICE;
             goto bail_out;
         }
+
+        r = crypt_keyslot_add_by_volume_key(luks_ctxt, 0,
+                                            (const char*)key, keylen,
+                                            passwd, strlen(passwd));
+        if (r < 0) {
+          fprintf(stderr, _("Failed to create LUKS key for \"%s\"\n"),
+                  boundtgt->tgt->ident);
+        }
     } else {
-        /* Changing password on existing partition */
         keyslot = (luksor != NULL ? luksor->keyslot : 0);
 
-        if (LUKS_read_phdr(lukshdr, 1, 0, luks_ctxt) < 0) {
-            fprintf(stderr, _("Failed to read LUKS header for \"%s\"\n"),
-                    boundtgt->tgt->ident);
-            eflag = ERR_BADDEVICE;
-            goto bail_out;
+        if (crypt_load(luks_ctxt, NULL, NULL) < 0) {
+          eflag = ERR_BADDEVICE;
+          goto bail_out;
         }
 
-        lukshdr->keyblock[keyslot].active = LUKS_KEY_DISABLED;
-    }
-
-    printf(_("Setting password on LUKS keyslot-%u\n"), keyslot);
-
-    if (LUKS_set_key(keyslot, passwd, strlen(passwd), lukshdr, lukskey, *iteration_time, PBKDF2_per_sec, luks_ctxt) < 0) {
-        char errmsg[80];
-        fprintf(stderr, _("Failed to create LUKS key for \"%s\"\n"),
-                boundtgt->tgt->ident);
-
-        crypt_last_error(luks_ctxt, errmsg, sizeof(errmsg));
-        fprintf(stderr, "LUKS error: %s\n", errmsg);
-        eflag = ERR_BADENCRYPT;
-        goto bail_out;
+        printf(_("Setting password on LUKS keyslot-%u\n"), keyslot);
+        if (kmluks_change_slot_passwd(luks_ctxt, keyslot,
+                                      key, keylen, passwd) < 0) {
+            char errmsg[256];
+            crypt_last_error(luks_ctxt, errmsg, sizeof(errmsg));
+            fprintf(stderr, "LUKS error: %s\n", errmsg);
+            eflag = ERR_BADENCRYPT;
+            goto bail_out;
+        }
     }
 
   bail_out:
 
-    unblockify_file(&tgtdev, killloop);
-    udev_settle();
-    if (lukskey != NULL) crypt_free_volume_key(lukskey);
+    crypt_free(luks_ctxt);
     if (passwd != NULL) sec_free((void*)passwd);
     if (ciphername != NULL) free((void*)ciphername);
     if (ciphermode != NULL) free((void*)ciphermode);
-    if (luks_ctxt != NULL) crypt_free(luks_ctxt);
+    udev_settle();
 
     return eflag;
-
-#else   /* !HAVE_UUID */
-
-    fprintf(stderr, _("Writing LUKS keys is not possible unless cryptmount has been built with the uuid-dev package installed\n"));
-
-    return ERR_BADKEYFORMAT;
-#endif  /* HAVE_UUID */
 }
 
 
