@@ -44,7 +44,8 @@
 #include "tables.h"
 #include "utils.h"
 #ifdef TESTING
-#  include <dirent.h>
+#  include <glob.h>
+#  include <libgen.h>
 #  include "cmtesting.h"
 #endif
 
@@ -348,8 +349,14 @@ bound_tgtdefn_t *bind_tgtdefn(const tgtdefn_t *tgt)
 #endif  /* !AS_MODULE */
 
 
+/*!
+ *  Extract the filesystem encryption key for a particular target.
+ *
+ *  This delegates most functionality to the methods defined
+ *  in the \ref keymanager_t structure.
+ */
 int cm_get_key(bound_tgtdefn_t *boundtgt, const km_pw_context_t *pw_ctxt,
-                unsigned char **key, int *keylen)
+               unsigned char **key, int *keylen)
 {   keyinfo_t *keyinfo = &boundtgt->tgt->key;
     FILE *fp = NULL;
     unsigned props, attempts = 0;
@@ -423,6 +430,9 @@ unsigned cm_get_keyproperties(const bound_tgtdefn_t *boundtgt)
 
 #ifdef TESTING
 
+/*! \addtogroup unit_tests
+ *  @{ */
+
 const char *km_legacy_key = "cryptmount012345";
 const char *km_legacy_passwd = "nothing";
 
@@ -573,15 +583,15 @@ int km_archive_keys(void)
 int km_test_legacy(void)
     /** Check that keyfiles from earlier releases can be read */
 {   keymanager_t *km;
-    DIR *dp;
-    const struct dirent *dent;
+    glob_t keyfiles;
     struct stat sbuff;
     tgtdefn_t *target;
     bound_tgtdefn_t *bound=NULL;
     km_pw_context_t pw_ctxt;
     unsigned char *key=NULL;
-    int n_legacy = 0, keylen, flg;
-    char *tokbuff, dirpath[1024], keypath[2048];
+    int idx, n_legacy = 0, keylen, flg;
+    char *tokbuff, keyglob[1024];
+    const char *basepath;
 
     CM_TEST_START("Legacy key support");
 
@@ -589,28 +599,27 @@ int km_test_legacy(void)
     pw_ctxt.verify = 0;
     pw_ctxt.argpasswd[0] = pw_ctxt.argpasswd[1] = km_legacy_passwd;
 
-    snprintf(dirpath, sizeof(dirpath), CM_SRCDIR "/testing/keys");
-    dp = opendir(dirpath);
-    if (dp == NULL) CM_TEST_FAIL();
+    snprintf(keyglob, sizeof(keyglob), CM_SRCDIR "/testing/keys/[0-9]*[0-9]");
+    glob(keyglob, 0, NULL, &keyfiles);
 
     for (km=keymgrs; km!=NULL; km=km->next) {
         km->initialized &= ~KM_TESTED;
     }
 
-    for (;;) {
-        dent = readdir(dp);
-        if (dent == NULL) break;
+    for (idx=0; idx<(int)keyfiles.gl_pathc; ++idx) {
+        const char *keypath = keyfiles.gl_pathv[idx];
 
-        snprintf(keypath, sizeof(keypath), "%s/%s", dirpath, dent->d_name);
         if (stat(keypath, &sbuff) != 0) CM_TEST_FAIL();
         if (!S_ISREG(sbuff.st_mode)) continue;
         ++n_legacy;
 
-        tokbuff = cm_strdup(dent->d_name);
+        strcpy(keyglob, keypath);
+        basepath = basename(keyglob);
+        tokbuff = cm_strdup(basepath);
         (void)strtok(tokbuff, "_");
 
         target = alloc_tgtdefn(NULL);
-        target->ident = cm_strdup(dent->d_name);
+        target->ident = cm_strdup(basepath);
         target->key.format = cm_strdup(strtok(NULL, "_"));
         target->key.filename = cm_strdup(keypath);
         target->key.digestalg = cm_strdup(strtok(NULL, "_"));
@@ -647,7 +656,7 @@ int km_test_legacy(void)
 
         free((void*)tokbuff);
     }
-    closedir(dp);
+    globfree(&keyfiles);
 
     if (n_legacy < 4) CM_TEST_FAIL();
 
@@ -662,6 +671,8 @@ int km_test_legacy(void)
     CM_TEST_OK();
 }
 
+/** @} */
+
 #endif  /* TESTING */
 
 
