@@ -77,7 +77,7 @@ typedef enum {
     M_PREPARE, M_RELEASE,
     M_MOUNT, M_UNMOUNT,
     M_SWAPON, M_SWAPOFF,
-    M_LIST, M_KEYMGRS,
+    M_LIST, M_KEYMGRS, M_STATUS,
     M_PASSWORD, M_KEYGEN, M_KEYREU,
     M_SYSBOOT, M_SYSQUIT,
     M_SAFETYNET, M_VERSION,
@@ -89,9 +89,9 @@ typedef enum {
  *  from command-line options
  */
 enum {
-    F_NEEDS_TGT =   1,
-    F_ALL_TARGETS = 2,
-    F_VERIFY_PW =   4
+    F_NEEDS_TGT =   1,    /*!< command requires a filesystem target */
+    F_ALL_TARGETS = 2,    /*!< command will be applied to all known targets */
+    F_VERIFY_PW =   4     /*!< encryption password should be double-checked */
 };
 
 
@@ -100,6 +100,7 @@ static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
                 const km_pw_context_t *pw_ctxt,
                 const char *params, const targelt_t *eltlist);
 static int do_list(const targelt_t *eltlist);
+static int do_status(const targelt_t *eltlist);
 static int do_keymgrlist();
 static int do_sysboot_updown(int booting, const tgtdefn_t *tgttable,
                 const km_pw_context_t *pw_ctxt);
@@ -126,6 +127,7 @@ usage: cryptmount [OPTION [target ...]]\n\
     -c | --change-password <target>\n\
     -k | --key-managers\n\
     -l | --list\n\
+    -S | --status\n\
     -m | --mount <target>\n\
     -u | --unmount <target>\n\
     --generate-key <key-size> <target>\n\
@@ -234,8 +236,12 @@ static int64_t getblk512count(const char *device, int *blklen)
 }
 
 
+/*! @brief Print list of available filing-system targets to stdout
+ *
+ *  This provides the back-end functionality for
+ *  the '--list' command-line option.
+ */
 static int do_list(const targelt_t *eltlist)
-    /** Print list of available filing-system targets to stdout */
 {   const targelt_t *elt;
 
     for (elt=eltlist; elt!=NULL; elt=elt->nx) {
@@ -254,8 +260,38 @@ static int do_list(const targelt_t *eltlist)
 }
 
 
-/**
- *  Print a list of all available key managers to stdout.
+/*! @brief Print mounting status of set of targets to stdout
+ *
+ *  This provides the back-end functionality for
+ *  the '--status' command-line option.
+ */
+static int do_status(const targelt_t *eltlist)
+{   const targelt_t *elt;
+    tgtstat_t *tgtstats = NULL, *ts;
+
+    tgtstats = get_all_tgtstatus();
+
+    for (elt=eltlist; elt!=NULL; elt=elt->nx) {
+        const tgtdefn_t *tgt = elt->tgt;
+
+        for (ts=tgtstats; ts!=NULL; ts=ts->nx) {
+          if (strcmp(ts->ident, tgt->ident) == 0) break;
+        }
+
+        printf("%-16s  %s\n",
+               tgt->ident, (ts != NULL ? "mounted" : "not_mounted"));
+    }
+
+    free_tgtstatus(tgtstats);
+
+    return ERR_NOERROR;
+}
+
+
+/*! @brief Print a list of all available key managers to stdout.
+ *
+ *  This provides the back-end functionality for
+ *  the '--key-managers' command-line option.
  */
 static int do_keymgrlist()
 {   const char **keymgrs = get_keymgr_list();
@@ -277,9 +313,15 @@ static int do_keymgrlist()
 }
 
 
+/*! @brief Setup all devices which have a bootaction option specified
+ *
+ *  This provides the back-end functionality for
+ *  the '--system-boot' and '--system-shutdown' command-line options.
+ *
+ *  \see do_mount(), do_swapon(), do_devsetup().
+ */
 static int do_sysboot_updown(int booting, const tgtdefn_t *tgttable,
                              const km_pw_context_t *pw_ctxt)
-    /** Setup all devices which have a bootaction option specified */
 {   const tgtdefn_t *tgt;
     bound_tgtdefn_t *boundtgt = NULL;
     int eflag = ERR_NOERROR;
@@ -315,9 +357,20 @@ static int do_sysboot_updown(int booting, const tgtdefn_t *tgttable,
 }
 
 
+/*! @brief Setup all devices needed to access encrypted target
+ *
+ *  This will wrap any file-based targets within a loopback device,
+ *  and then setup the device-mapper to provide encryption/decryption
+ *  of the target block device after obtaining the access password
+ *  from the user.
+ *
+ *  This provides the back-end functionality for
+ *  the '--prepare' command-line option.
+ *
+ *  \see cm_get_key(), blockify_file(), devmap_create(), do_devshutdown().
+ */
 static int do_devsetup(const km_pw_context_t *pw_ctxt,
                        bound_tgtdefn_t *boundtgt, char **mntdev)
-    /** Setup all devices needed to access encrypted target */
 {   enum { BUFFMIN=1024 };
     unsigned char *key = NULL;
     int buffpos, blklen, readonly, isloop = 0, killloop = 0,
@@ -405,8 +458,17 @@ static int do_devsetup(const km_pw_context_t *pw_ctxt,
 }   /* do_devsetup() */
 
 
+/*! @brief Remove all devices attached to encrypted target
+ *
+ *  This will close-down device-mapper and loopback devices
+ *  configured by do_setup().
+ *
+ *  This provides the back-end functionality for
+ *  the '--release' command-line option.
+ *
+ *  \see do_devsetup(), devmap_remove(), loop_dellist().
+ */
 int do_devshutdown(const bound_tgtdefn_t *boundtgt)
-    /** Remove all devices attached to encrypted target */
 {   const tgtdefn_t *tgt = boundtgt->tgt;
     struct stat sbuff;
     unsigned devcnt=0;
@@ -457,6 +519,13 @@ int do_devshutdown(const bound_tgtdefn_t *boundtgt)
 }
 
 
+/*! @brief Mount an encrypted filesystem
+ *
+ *  This provides the back-end functionality for
+ *  the '--mount' command-line option.
+ *
+ *  \see do_unmount(), do_devsetup(), fs_mount().
+ */
 static int do_mount(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
 {   const tgtdefn_t *tgt=NULL;
     int freedev=0, eflag=ERR_NOERROR;
@@ -506,6 +575,13 @@ static int do_mount(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
 }
 
 
+/*! @brief Unmount an encrypted filesystem
+ *
+ *  This provides the back-end functionality for
+ *  the '--unmount' command-line option.
+ *
+ *  \see do_mount(), do_devshutdown(), fs_unmount().
+ */
 static int do_unmount(const bound_tgtdefn_t *boundtgt)
 {   const tgtdefn_t *tgt = boundtgt->tgt;
     int eflag=ERR_NOERROR;
@@ -559,6 +635,13 @@ static int do_unmount(const bound_tgtdefn_t *boundtgt)
 }
 
 
+/*! @brief Setup an encrypted swap partition
+ *
+ *  This provides the back-end functionality for
+ *  the '--swapon' command-line option.
+ *
+ *  \see do_swapoff(), do_devsetup(), fs_swapon().
+ */
 static int do_swapon(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
 {   const tgtdefn_t *tgt=NULL;
     int freedev=0, eflag=ERR_NOERROR;
@@ -603,6 +686,13 @@ static int do_swapon(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
 }
 
 
+/*! @brief Close down an encrypted swap partition
+ *
+ *  This provides the back-end functionality for
+ *  the '--swapoff' command-line option.
+ *
+ *  \see do_swapon(), do_devshutdown(), fs_swapoff().
+ */
 static int do_swapoff(const bound_tgtdefn_t *boundtgt)
 {   const tgtdefn_t *tgt = boundtgt->tgt;
     int eflag=ERR_NOERROR;
@@ -646,16 +736,22 @@ static int do_swapoff(const bound_tgtdefn_t *boundtgt)
 }
 
 
+/*! @brief Change access password on particular target
+ *
+ *  This provides the back-end functionality for
+ *  the '--change-password' command-line option.
+ *
+ *  \see cm_get_key(), cm_put_key().
+ */
 static int do_passwd(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
-    /** Change password on particular target */
 {   tgtdefn_t *tgt = boundtgt->tgt;
-    unsigned char *key=NULL;
+    unsigned char *key = NULL;
     unsigned keyprops;
-    int keylen=0, eflag=ERR_NOERROR;
-    char *newfname=NULL, *oldfname=NULL;
+    int keylen = 0, eflag = ERR_NOERROR;
+    char *newfname = NULL, *oldfname = NULL;
     struct stat sbuff;
     size_t sz;
-    FILE *fp=NULL;
+    FILE *fp = NULL;
 
     keyprops = cm_get_keyproperties(boundtgt);
     if ((keyprops & KM_PROP_HASPASSWD) == 0) {
@@ -729,9 +825,15 @@ static int do_passwd(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
 }
 
 
+/*! @brief Create new filesystem crypto-key
+ *
+ *  This provides the back-end functionality for
+ *  the '--generate-key' command-line option.
+ *
+ *  \see cm_generate_key(), cm_put_key().
+ */
 static int do_keygen(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt,
                     const char *params, int reuse, const tgtdefn_t *tgttable)
-    /** Create new filesystem crypto-key */
 {   unsigned char *key=NULL;
     unsigned keyprops;
     int keylen=0, fileexists=0, eflag=ERR_NOERROR;
@@ -835,8 +937,21 @@ static int do_keygen(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt,
 }
 
 
+/*! @brief Attempt to unmount/shutdown all targets currently mounted.
+ *
+ *  This will identify all filesystems listed in cryptmount's own
+ *  mounting table (typically /run/cryptmount.status), and try
+ *  to unmount, or de-configure, any targets that seem still to be in use.
+ *
+ *  This will generally only be useful during a system shutdown,
+ *  to provide a safety mechanism for filesystems that have
+ *  not been unmounted normally. Accordingly, this routine
+ *  is more aggressive in its approach than do_unmount(), do_swapoff(), etc.
+ *
+ *  This provides the back-end functionality for
+ *  the '--safetynet' command-line option.
+ */
 static int do_safetynet()
-    /** Last-ditch attempt to unmount/shutdown all targets currently mounted */
 {   tgtstat_t *all_tsts=NULL, *tst;
     char *devname=NULL;
     const char *old_ident=NULL;
@@ -958,10 +1073,16 @@ static int check_priv_tgt(const tgtdefn_t *tgt)
 }
 
 
+/*! @brief Apply top-level mode-dependent operation to list of targets
+ *
+ *  This will mount/unmount/swapon/swapoff the give set of encrypted
+ *  filesystems, according to an application-level choice of operating mode.
+ *
+ *  \see do_mount(), do_swapon(), do_list(), do_safetynet(), parse_options().
+ */
 static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
                 const km_pw_context_t *pw_ctxt,
                 const char *params, const targelt_t *eltlist)
-    /** Apply mode-dependent operation to list of targets */
 {   const targelt_t *elt = NULL;
     bound_tgtdefn_t *boundtgt = NULL;
     int ignore_eltlist = 1, prio = 0, eflag = ERR_NOERROR;
@@ -986,6 +1107,9 @@ static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
         case M_LIST:
             do_list(eltlist);
             break;
+        case M_STATUS:
+            do_status(eltlist);
+            break;
         case M_SYSBOOT:
             do_sysboot_updown(1, tgttable, pw_ctxt);
             break;
@@ -1126,9 +1250,9 @@ static cmmode_t get_defaultmode(int argc, char *argv[])
  *  only accessible to the super-user.
  */
 cmmode_t parse_options(int argc, char *argv[],
-                        unsigned *flags, const char **mode_params,
-                        int *passwd_fd, int *config_fd,
-                        km_pw_context_t *pw_ctxt)
+                       unsigned *flags, const char **mode_params,
+                       int *passwd_fd, int *config_fd,
+                       km_pw_context_t *pw_ctxt)
 {   cmmode_t mode = M_UNSET;
     enum {
         ALL_USERS =     0x01,
@@ -1173,6 +1297,8 @@ cmmode_t parse_options(int argc, char *argv[],
                                     mode_params, M_KEYREU, F_NEEDS_TGT },
         { 'n', "safetynet",         SET_MODE,
                                     NULL, M_SAFETYNET, 0 },
+        { 'S', "status",            ALL_USERS | SET_MODE | SET_FLAGS,
+                                    NULL, M_STATUS },
         { 's', "swapon",            SET_MODE | SET_FLAGS,
                                     NULL, M_SWAPON, F_NEEDS_TGT },
         { 'x', "swapoff",           SET_MODE | SET_FLAGS,
@@ -1360,7 +1486,9 @@ int main(int argc, char *argv[])
     }
     free((void*)cmtab);
 
-    if (mode == M_LIST && optind >= argc) mode_flags |= F_ALL_TARGETS;
+    if ((mode == M_LIST || mode == M_STATUS) && optind >= argc) {
+        mode_flags |= F_ALL_TARGETS;
+    }
 
     /* if '--all' given, assemble list of targets from entire config-file */
     if ((mode_flags & F_ALL_TARGETS) != 0) {
