@@ -24,16 +24,26 @@
 #ifndef _TABLES_H
 #define _TABLES_H
 
+/*! \addtogroup cmtab_utils
+ *  @{ */
+
 extern const char *cm_status_filename;
 
 struct tgtdefn;
 
+/*!
+ *  Record for persistent storage of target status.
+ *
+ *  This is used to populate entries in /run/cryptmount.status
+ *  where each target that is either mounted or active as a swap device
+ *  is listed together with the user-id of its owner.
+ */
 typedef struct tgtstat
 {
-    char *ident;            /*!< unique identifying name of target */
-    unsigned long uid;      /*!< user-ID responsible for mounting filesystem */
+    char *ident;            /*!< Unique identifying name of target */
+    unsigned long uid;      /*!< User-ID responsible for mounting filesystem */
 
-    struct tgtstat *nx;     /*!< form into linked list */
+    struct tgtstat *nx;     /*!< Form into linked list */
 } tgtstat_t;
 
 
@@ -56,6 +66,8 @@ int put_tgtstatus(const struct tgtdefn *tgt, const tgtstat_t *tstat);
 void free_tgtstatus(tgtstat_t *tstat);
 int is_cmstatus_intact();
 
+/** @} */
+
 #endif  /* _TABLES_H */
 
 /*
